`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company:     ARM
// Engineer:    Jack Pinnington
// 
// Create Date: 22.08.2019 16:31:37
// Design Name: 
// Module Name: AHB2DUALSPI
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module AHB2DUALSPI(
    //AHBLITE INTERFACE
    //Slave Select Signals
    input   wire            HSEL,
    //Global Signal
    input   wire            HCLK,
    input   wire            HRESETn,
    //Address, Control & Write Data
    input   wire            HREADY,
    input   wire    [31:0]  HADDR,
    input   wire    [1:0]   HTRANS,
    input   wire            HWRITE,
  
    input   wire    [31:0]  HWDATA,
    // Transfer Response & Read Data
    output  wire            HREADYOUT,
    output  wire     [31:0]  HRDATA,
    
    //SPI buses for 2 SPI devices
    input                   SPI_MISO,
    output                  SPI_MOSI,
    output                  SPI_SCLK,
    output                  SPI_SS
    );
    
 assign HREADYOUT = 1'b1; // Always ready - waiting for command is provided using the rw_control_reg.
 
/*Registers for the AHB read/write.*/
localparam COMMAND_ADDR = 8'h00;
localparam X_ADDR = 8'h04;
localparam Y_ADDR = 8'h08;
localparam STATUS_ADDR = 8'h0C;
localparam RW_CONTROL_ADDR = 8'h10;

reg     [7:0]   write_command;
wire    [15:0]  x_smp;
wire    [15:0]  y_smp;
wire    [7:0]   status;
reg     [7:0]   rw_control_reg;

// Registers to store Adress Phase Signals
reg APhase_HSEL;
reg APhase_HWRITE;
reg [1:0] APhase_HTRANS;
reg [31:0] APhase_HRADDR;
reg [31:0] APhase_HWADDR;

// Sample the Address Phase   
always @(posedge HCLK or negedge HRESETn)
    begin
    if(!HRESETn)
        begin
        APhase_HSEL <= 1'b0;
        APhase_HWRITE <= 1'b0;
        APhase_HTRANS <= 2'b00;
        APhase_HWADDR <= 32'h0;
        APhase_HRADDR <= 32'h0000_0000;
        end
    else if(HREADY)
        begin
        APhase_HSEL <= HSEL;
        APhase_HWRITE <= HWRITE;
        APhase_HTRANS <= HTRANS;
        APhase_HWADDR <= HADDR;
        APhase_HRADDR <= HADDR;
        end
    end
   
/*Assign the read data.*/
assign HRDATA = (APhase_HRADDR[7:0] == COMMAND_ADDR) ? {24'h0000_00, write_command}:
                  (APhase_HRADDR[7:0] == X_ADDR) ? {16'h0000, x_smp} :
                  (APhase_HRADDR[7:0] == Y_ADDR) ? {24'h0000_00, y_smp} :
                  (APhase_HRADDR[7:0] == STATUS_ADDR) ? {16'h0000, status} :
                  (APhase_HRADDR[7:0] == RW_CONTROL_ADDR) ? {24'h0000_00, rw_control_reg}:
                   32'h0000_0000;
    
always @ (posedge HCLK)
    begin
    if(APhase_HSEL & APhase_HWRITE & APhase_HTRANS[1])
        begin
        case(APhase_HWADDR[7:0])
           COMMAND_ADDR: write_command <= HWDATA[7:0];
           RW_CONTROL_ADDR: rw_control_reg[0] <= HWDATA[0];
        endcase
        end
    end
    
// ===========================================================================
// 							  Parameters, Regsiters, and Wires
// ===========================================================================

// Output wires and registers
wire SS;
wire MOSI;
wire [39:0] DOUT;

wire getByte;									// Initiates a data byte transfer in SPI_Int
wire [7:0] sndData;							// Data to be sent to Slave
wire [7:0] RxData;							// Output data from SPI_Int
wire BUSY;										// Handshake from SPI_Int to SPI_Ctrl


// 66.67kHz Clock Divider, period 15us
reg iSCLK;										// Internal serial clock,
                                                    // not directly output to slave,
                                                    // controls state machine, etc.
    
// ===========================================================================
// 										Implementation
// ===========================================================================

//-----------------------------------------------
//  	  				SPI Controller
//-----------------------------------------------
spiCtrl SPI_Ctrl(
        .CLK(iSCLK),
        .RST(!HRESETn),
        .sndRec(rw_control_reg[0]),
        .BUSY(BUSY),
        .DIN(write_command),
        .RxData(RxData),
        .SS(SPI_SS),
        .getByte(getByte),
        .sndData(sndData),
        .DOUT(DOUT)
);

//-----------------------------------------------
//  	  				  SPI Mode 0
//-----------------------------------------------
spiMode0 SPI_Int(
        .CLK(iSCLK),
        .RST(!HRESETn),
        .sndRec(getByte),
        .DIN(sndData),
        .MISO(SPI_MISO),
        .MOSI(SPI_MOSI),
        .SCLK(SPI_SCLK),
        .BUSY(BUSY),
        .DOUT(RxData)
);

//Generate the 66.67KHz SPI clock.
parameter spiClkCountVal = 375;
reg [9:0] spiClkCounter = 10'b0000000000;

always @(posedge HCLK) 
    begin
    if(!HRESETn) 
        begin
        iSCLK <= 1'b0;
        spiClkCounter <= 10'b0000000000;
        end
    else 
        begin
        if(spiClkCounter == spiClkCountVal) 
            begin
            iSCLK <= ~iSCLK;
            spiClkCounter <= 10'b0000000000;
            end
        else 
            begin
            spiClkCounter <= spiClkCounter + 1'b1;
            end
        end
    end

assign x_smp = {DOUT[25:24], DOUT[39:32]};//{DOUT[9:8], DOUT[23:16]};
assign y_smp = {DOUT[9:8], DOUT[23:16]};
assign status = DOUT[7:0];
// Assign the busy register using the Busy handshake from the SPI Controller
			
always @ (posedge HCLK)
    begin
    rw_control_reg[1] <= BUSY;
    end
    
endmodule
